@@ -1,43 +1,12 @@
 import operator
-from fnmatch import fnmatchcase
 from functools import reduce
 from functools import wraps
-from itertools import chain
 
 from .matching import match_style
-from .utils import _stringlike
 from .utils import pathlike
 
 
-def _non_stringlike_iterable(obj):  # TODO rename to _non_stringlike_iterable
-    """Guess if an object could be a nested collection.
-
-    The general heuristic is that an object must be iterable, but not stringlike
-    so that an element can be arbitrary, like another collection.
-
-    Since this is a simple check, there are false positives, but this works with
-    basic types.
-    >>> _non_stringlike_iterable([1])
-    True
-    >>> _non_stringlike_iterable({1:2})
-    True
-    >>> _non_stringlike_iterable(1)
-    False
-    >>> _non_stringlike_iterable("a")
-    False
-    """
-    try:
-        iter(obj)
-    except TypeError:
-        return False
-
-    if _stringlike(obj):
-        return False
-
-    return True
-
-
-def del_by_path(obj, path):
+def del_by_path(obj, path, match_with="glob", recursive_match_all=True, **kwargs):
     """Delete a key-value in a nested object in root by item sequence.
     from https://stackoverflow.com/a/14692747/913080
 
@@ -46,7 +15,7 @@ def del_by_path(obj, path):
     >>> obj
     {'a': [{'c': 'd'}]}
     """
-    del getitem_by_path(obj, path[:-1])[path[-1]]
+    del getitem_by_path(obj, path[:-1], match_with, recursive_match_all, **kwargs)[path[-1]]
 
 
 def getitem_by_path_strict(obj, path):
@@ -87,13 +56,6 @@ def get_by_path_strict(obj, path, default=None):
         return default
 
 
-def _match(a, b):
-    try:
-        return fnmatchcase(a, b)
-    except TypeError:  # fallback to normal eq
-        return a == b
-
-
 def _simplify_double_splats(path):
     """Return an equivalent path, removing any unnecessary double splats."""
     # remove ending "**"
@@ -114,7 +76,7 @@ def _simplify_double_splats(path):
     return path
 
 
-def resolve_path(obj, path, match_with="glob", recursive_match_all=True):
+def resolve_path(obj, path, *args, match_with="glob", recursive_match_all=True, **kwargs):
     """Yield all paths that match the given globbed path."""
     if recursive_match_all and "**" in path:
         path = _simplify_double_splats(path)
@@ -127,35 +89,69 @@ def resolve_path(obj, path, match_with="glob", recursive_match_all=True):
         path_end = path[double_splat_idx:]
 
         if path_start:
-            paths = list(resolve_path(obj, path_start))
+            paths = list(
+                resolve_path(
+                    obj,
+                    path_start,
+                    *args,
+                    match_with=match_with,
+                    recursive_match_all=recursive_match_all,
+                    **kwargs,
+                )
+            )
             for p in paths:
-                path_ends = list(paths_to_key(getitem_by_path_strict(obj, p), path_end))
+                path_ends = list(
+                    paths_to_key(
+                        getitem_by_path_strict(obj, p),
+                        path_end,
+                        *args,
+                        match_with=match_with,
+                        recursive_match_all=recursive_match_all,
+                        **kwargs,
+                    )
+                )
                 for end in path_ends:
                     yield p + end
         else:  # path started with "**"
-            path_ends = list(paths_to_key(obj, path_end))
+            path_ends = list(
+                paths_to_key(
+                    obj,
+                    path_end,
+                    *args,
+                    match_with=match_with,
+                    recursive_match_all=recursive_match_all,
+                    **kwargs,
+                )
+            )
             yield from path_ends
     elif path:
         first_step = path[0]
         path_remainder = path[1:]
 
-        keys = matched_keys(obj, first_step, match_with)
+        keys = matched_keys(obj, first_step, *args, match_with=match_with, **kwargs)
 
         if path_remainder:
             for key in keys:
                 sub_obj = obj[key]
-                smk = matched_keys(sub_obj, path_remainder[0], match_with)
+                smk = matched_keys(sub_obj, path_remainder[0], match_with, **kwargs)
                 if smk:
                     yv = (
                         [key] + sk
-                        for sk in resolve_path(sub_obj, path_remainder, match_with)
+                        for sk in resolve_path(
+                            sub_obj,
+                            path_remainder,
+                            *args,
+                            match_with=match_with,
+                            recursive_match_all=recursive_match_all,
+                            **kwargs,
+                        )
                     )
                     yield from yv
         else:
             yield from ([key] for key in keys)
 
 
-def matched_keys(obj, pattern, match_with="glob"):
+def matched_keys(obj, pattern, *args, match_with="glob", **kwargs):
     """
     >>> matched_keys(1, '0')
     []
@@ -182,22 +178,22 @@ def matched_keys(obj, pattern, match_with="glob"):
 
     try:
         for key in obj.keys():
-            if match_func(key, pattern):
+            if match_func(key, pattern, *args, **kwargs):
                 rv.append(key)
     except AttributeError:
         for idx in range(len(obj)):
-            if match_func(idx, pattern):
+            if match_func(idx, pattern, *args, **kwargs):
                 rv.append(idx)
     return rv
 
 
-def getitem_by_path(obj, path, match_with="glob"):
+def getitem_by_path(obj, path, *args, match_with="glob", **kwargs):
     if not pathlike(path):  # e.g. str or int
-        if not match_style(match_with).patterned(path):
+        if not match_style(match_with).patterned(path, *args, **kwargs):
             return obj[path]
         path = [path]
 
-    paths = list(resolve_path(obj, path, match_with))
+    paths = list(resolve_path(obj, path, *args, match_with=match_with, **kwargs))
 
     if len(paths) > 1:
         return [getitem_by_path_strict(obj, p) for p in paths]
@@ -207,13 +203,13 @@ def getitem_by_path(obj, path, match_with="glob"):
     # Check if any part of the path appears to use a pattern. If so, return an empty
     # list, rather than a KeyError/IndexError because 0 matched results.
 
-    if any(match_style(match_with).patterned(p) for p in path):
+    if any(match_style(match_with).patterned(p, *args, **kwargs) for p in path):
         return paths
 
     return getitem_by_path_strict(obj, path)
 
 
-def set_by_path(obj, path, value):
+def set_by_path(obj, path, value, *args, match_with="glob", recursive_match_all=True, **kwargs):
     """Set a value in a nested object in obj by iterable path.
 
     If the path doesn't fully exist, this will create it to set the value,
@@ -231,12 +227,20 @@ def set_by_path(obj, path, value):
     >>> obj
     {0: {1: {2: {3: {4: {5: {6: {7: {8: {9: 10}}}}}}}}}}
     """
-    path = list(path)  # needed for later equality checks
+    # needed for later equality checks. In general paths don't need to be lists.
+    path = list(path)
 
     traversed = []
     for part in path:
         # branch = get_by_path(obj, [part])
-        branch = getitem_by_path(obj, traversed)
+        branch = getitem_by_path(
+            obj,
+            traversed,
+            *args,
+            match_with=match_with,
+            recursive_match_all=recursive_match_all,
+            **kwargs,
+        )
         traversed.append(part)
 
         try:
@@ -248,7 +252,71 @@ def set_by_path(obj, path, value):
             branch[part] = value
 
 
-def paths_to_key(obj, key, current=None):
+def _paths_to_pathlike_key(obj, key, *args, match_with, _current, **kwargs):
+    if len(key) > 1:
+        resolved_paths = list(resolve_path(obj, key, *args, match_with=match_with, **kwargs))
+        if resolved_paths:
+            for path in resolved_paths:
+                yield _current + path
+        else:
+            try:
+                for k, v in obj.items():
+                    if pathlike(v):
+                        yield from paths_to_key(
+                            v,
+                            key,
+                            *args,
+                            match_with=match_with,
+                            _current=_current + [k],
+                            **kwargs,
+                        )
+            except AttributeError:
+                for idx, i in enumerate(obj):
+                    if pathlike(i):
+                        yield from paths_to_key(
+                            i,
+                            key,
+                            *args,
+                            match_with=match_with,
+                            _current=_current + [idx],
+                            **kwargs,
+                        )
+    else:
+        yield from paths_to_key(
+            obj,
+            next(iter(key)),
+            *args,
+            match_with=match_with,
+            _current=_current,
+            **kwargs,
+        )
+
+
+def _paths_to_simple_key(obj, key, *args, match_with, _current, **kwargs):
+    match_func = match_style(match_with).match
+    try:
+        for k, v in obj.items():
+            if pathlike(v):
+                yield from paths_to_key(v, key, match_with=match_with, _current=_current + [k], **kwargs)
+            if match_func(k, key):
+                yield _current + [k]
+    except AttributeError:  # no .items
+        for idx, v in enumerate(obj):
+            if pathlike(v):
+                yield from paths_to_key(v, key, match_with=match_with, _current=_current + [idx], **kwargs)
+            elif match_func(idx, key):
+                yield _current + [idx]
+
+
+def paths_to_key(
+    obj,
+    key,
+    *args,
+    match_with="glob",
+    recursive_match_all=True,
+    _current=None,
+    **kwargs,
+):
     """Return the path to a specified key in an object.
     A key may be simple or iterable, e.g. a str, int, list, dict, etc, as long as it
     can be used with one of the supported pattern matches,
@@ -276,48 +344,73 @@ def paths_to_key(obj, key, current=None):
     >>> list(paths_to_key({"x": 0}, ["y"]))
     []
     """
-    if current is None:
-        current = []
+    if not pathlike(obj):
+        raise TypeError(f"First argument must be able to be deep, not type '{type(obj)}'")
 
-    if not _non_stringlike_iterable(obj):
-        raise TypeError(
-            f"First argument must be able to be deep, not type '{type(obj)}'"
+    if _current is None:
+        _current = []
+
+    if pathlike(key):
+        yield from _paths_to_pathlike_key(
+            obj,
+            key,
+            *args,
+            match_with=match_with,
+            _current=_current,
+            recursive_match_all=recursive_match_all,
+            **kwargs,
+        )
+    else:  # key is simple; str, int, float, etc.
+        yield from _paths_to_simple_key(
+            obj,
+            key,
+            *args,
+            match_with=match_with,
+            _current=_current,
+            recursive_match_all=recursive_match_all,
+            **kwargs,
         )
 
-    compound_key = _non_stringlike_iterable(key)
-    if compound_key and len(key) > 1:
-        resolved_paths = list(resolve_path(obj, key))
-        if resolved_paths:
-            for key in resolved_paths:
-                yield current + key
-        else:
-            try:
-                for k, v in obj.items():
-                    if _non_stringlike_iterable(v):
-                        yield from paths_to_key(v, key, current + [k])
-            except AttributeError:
-                for idx, i in enumerate(obj):
-                    if _non_stringlike_iterable(i):
-                        yield from paths_to_key(i, key, current + [idx])
-    elif compound_key:
-        for k in key:  # Only iterates once, but works for dict or list-like keys
-            yield from paths_to_key(obj, k)
-    else:  # key is simple; str, int, float, etc.
-        try:
-            for k, v in obj.items():
-                if _non_stringlike_iterable(v):
-                    yield from paths_to_key(v, key, current + [k])
-                if _match(k, key):
-                    yield current + [k]
-        except AttributeError:  # no .items
-            for idx, v in enumerate(obj):
-                if _non_stringlike_iterable(v):
-                    yield from paths_to_key(v, key, current + [idx])
-                elif _match(idx, key):
-                    yield current + [idx]
+
+def _paths_to_pathlike_value(obj, value, *args, match_with, _current, **kwargs):
+    # Don't test non-pathlike obj elements since they can't match a pathlike value.
+    # Being pathlike is a proxy test for being deep. If something isn't pathlike, it can't be deep.
+    try:
+        for k, v in obj.items():
+            if pathlike(v):
+                yield from paths_to_value(v, value, *args, match_with=match_with, _current=_current + [k], **kwargs)
+    except AttributeError:
+        for idx, i in enumerate(obj):
+            if pathlike(i):
+                yield from paths_to_value(i, value, *args, match_with=match_with, _current=_current + [idx], **kwargs)
 
 
-def paths_to_value(obj, value, current=None):
+def _paths_to_simple_value(obj, value, *args, match_with, _current, **kwargs):
+    match_func = match_style(match_with).match
+
+    try:
+        for k, v in obj.items():
+            if pathlike(v):
+                yield from paths_to_value(v, value, *args, match_with=match_with, _current=_current + [k], **kwargs)
+            if match_func(v, value, *args, **kwargs):
+                yield _current + [k]
+    except AttributeError:  # no .items
+        for idx, i in enumerate(obj):
+            if pathlike(i):
+                yield from paths_to_value(i, value, *args, match_with=match_with, _current=_current + [idx], **kwargs)
+            elif match_func(i, value, *args, **kwargs):
+                yield _current + [idx]
+
+
+def paths_to_value(
+    obj,
+    value,
+    *args,
+    match_with="glob",
+    recursive_match_all=True,
+    _current=None,
+    **kwargs,
+):
     """Return the path to a specified value in an object.
     A value may be simple or iterable, e.g. a str, int, list, dict, etc, as long as it
     can be used with one of the supported pattern matches,
@@ -328,53 +421,61 @@ def paths_to_value(obj, value, current=None):
     >>> list(paths_to_value(["value"], "value"))
     [[0]]
     """
-    if current is None:
-        current = []
+    if not pathlike(obj):
+        raise TypeError(f"First argument must be able to be deep, not type '{type(obj)}'")
 
-    if not _non_stringlike_iterable(obj):
-        raise TypeError(
-            f"First argument must be able to be deep, not type '{type(obj)}'"
+    match_func = match_style(match_with).match
+
+    if _current is None:
+        _current = []
+
+    if match_func(obj, value, *args, **kwargs):
+        yield _current
+    # if no match, recurse
+    elif pathlike(value):  # e.g. list, dict. Can be another path
+        yield from _paths_to_pathlike_value(
+            obj,
+            value,
+            *args,
+            match_with=match_with,
+            _current=_current,
+            recursive_match_all=recursive_match_all,
+            **kwargs,
+        )
+    else:  # value is simple; str, int, float, etc.
+        yield from _paths_to_simple_value(
+            obj,
+            value,
+            *args,
+            match_with=match_with,
+            _current=_current,
+            recursive_match_all=recursive_match_all,
+            **kwargs,
         )
 
-    if _match(obj, value):
-        yield current
-    elif _non_stringlike_iterable(value):  # e.g. list, dict. Can be another path
-        try:
-            gbp = getitem_by_path(obj, value)
-        except (KeyError, IndexError, TypeError):
-            try:
-                for k, v in obj.items():
-                    if _non_stringlike_iterable(v):
-                        yield from paths_to_value(v, value, current + [k])
-            except AttributeError:
-                for idx, i in enumerate(obj):
-                    if _non_stringlike_iterable(i):
-                        yield from paths_to_value(i, value, current + [idx])
-        else:
-            yield current + list(value)
-    else:  # value is simple; str, int, float, etc.
-        try:
-            for k, v in obj.items():
-                if _non_stringlike_iterable(v):
-                    yield from paths_to_value(v, value, current + [k])
-                if _match(v, value):
-                    yield current + [k]
-        except AttributeError:  # no .items
-            for idx, i in enumerate(obj):
-                if _non_stringlike_iterable(i):
-                    yield from paths_to_value(i, value, current + [idx])
-                elif _match(i, value):
-                    yield current + [idx]
 
-
-def values_for_key(obj, key):
+def values_for_key(obj, key, *args, match_with="glob", recursive_match_all=True, **kwargs):
     """Generate all values for a given key.
 
     >>> list(values_for_key([{"x": {"y": "value", "z": {"y": "value"}}, "y": {1: 2}}], "y"))
     ['value', 'value', {1: 2}]
     """
-    for path in paths_to_key(obj, key):
-        yield getitem_by_path(obj, path)
+    for path in paths_to_key(
+        obj,
+        key,
+        *args,
+        match_with=match_with,
+        recursive_match_all=recursive_match_all,
+        **kwargs,
+    ):
+        yield getitem_by_path(
+            obj,
+            path,
+            *args,
+            match_with=match_with,
+            recursive_match_all=recursive_match_all,
+            **kwargs,
+        )
 
 
 def deduped_items(items):
@@ -399,8 +500,13 @@ class DynamicSubclasser(type):
     and assume that argument's class as its parent class.
 
     >>> class Foo(metaclass=DynamicSubclasser): pass
-    >>> foo = Foo(5)
-    >>> isinstance(foo, int)
+    >>> foo = Foo([5])
+    >>> foo == [5]
+    True
+    >>> isinstance(foo, list)
+    True
+    >>> foo = Foo()
+    >>> foo == {}
     True
     """
 
@@ -488,7 +594,17 @@ class DeepCollection(metaclass=DynamicSubclasser):
     'j'
     """
 
-    def __init__(self, obj, *args, return_deep=True, **kwargs):
+    def __init__(
+        self,
+        obj,
+        *args,
+        return_deep=True,
+        match_with="glob",
+        recursive_match_all=True,
+        match_args=None,
+        match_kwargs=None,
+        **kwargs,
+    ):
         # Set instance vars first in case anything else (like super().__init__) accesses
         # methods that trigger our __getattribute__, which needs them present.
         #
@@ -503,6 +619,10 @@ class DeepCollection(metaclass=DynamicSubclasser):
 
         self.original_type = type(self._obj)
         self.return_deep = return_deep
+        self.match_with = match_with
+        self.recursive_match_all = recursive_match_all
+        self.match_args = match_args or ()
+        self.match_kwargs = match_kwargs or {}
 
         # This often sets the original value for `self` for mutable types.
         # I.e. it gives a new list its content.
@@ -567,11 +687,7 @@ class DeepCollection(metaclass=DynamicSubclasser):
         >>> dc._obj
         [1, 2]
         """
-        if (
-            not name.startswith("_")
-            and name not in dir(DeepCollection)
-            and name in dir(self)
-        ):
+        if not name.startswith("_") and name not in dir(DeepCollection) and name in dir(self):
             method = object.__getattribute__(self, name)
             if callable(method):
                 # wrapped
@@ -591,22 +707,50 @@ class DeepCollection(metaclass=DynamicSubclasser):
         # Use self._obj instead of self to avoid unnecessary intermediate
         # DeepCollections. Just make a final conversion at the end.
 
-        rv = getitem_by_path(self._obj, path)
+        rv = getitem_by_path(
+            self._obj,
+            path,
+            *self.match_args,
+            match_with=self.match_with,
+            recursive_match_all=self.recursive_match_all,
+            **self.match_kwargs,
+        )
 
-        if _non_stringlike_iterable(rv) and self.return_deep:
-            return DeepCollection(rv)
+        if pathlike(rv) and self.return_deep:
+            return DeepCollection(
+                rv,
+                match_with=self.match_with,
+                recursive_match_all=self.recursive_match_all,
+                match_args=self.match_args,
+                match_kwargs=self.match_kwargs,
+            )
         return rv
 
     def __delitem__(self, path):
-        if _non_stringlike_iterable(path):
-            del_by_path(self, path)
+        if pathlike(path):
+            del_by_path(
+                self,
+                path,
+                *self.match_args,
+                match_with=self.match_with,
+                recursive_match_all=self.recursive_match_all,
+                **self.match_kwargs,
+            )
         else:
             super().__delitem__(path)
             del self._obj[path]
 
     def __setitem__(self, path, value):
-        if _non_stringlike_iterable(path):
-            set_by_path(self, path, value)
+        if pathlike(path):
+            set_by_path(
+                self,
+                path,
+                value,
+                *self.match_args,
+                match_with=self.match_with,
+                recursive_match_all=self.recursive_match_all,
+                **self.match_kwargs,
+            )
         else:
             super().__setitem__(path, value)
             self._obj[path] = value
@@ -615,39 +759,78 @@ class DeepCollection(metaclass=DynamicSubclasser):
         super_repr = super().__repr__()
         # Some collections types already display self when self isn't the original type,
         # but that's actually not what we want here, so if we find that, fix it.
-        super_repr = super_repr.replace(
-            self.__class__.__name__, self.original_type.__name__
-        )
+        super_repr = super_repr.replace(self.__class__.__name__, self.original_type.__name__)
         return f"DeepCollection({super_repr})"
 
     # Common public methods
-    def get(self, path, default=None):
+    def get(self, path, default=None, *, match_args=None, match_with=None, recursive_match_all=None, match_kwargs=None):
+        # These are one-offs and should not mutate self
+        match_args = match_args or self.match_args
+        match_with = match_with or self.match_with
+        match_kwargs = match_kwargs or self.match_kwargs
+        if recursive_match_all is None:
+            recursive_match_all = self.recursive_match_all
+
         try:
-            return self.__getitem__(path)
+            rv = getitem_by_path(
+                self._obj,
+                path,
+                *match_args,
+                match_with=match_with,
+                recursive_match_all=recursive_match_all,
+                **match_kwargs,
+            )
         except (KeyError, IndexError, TypeError):
             return default
 
+        if pathlike(rv) and self.return_deep:  # pathlike is a proxy test for an object being deepable
+            # retain settings from self, not the one-off values
+            return DeepCollection(
+                rv,
+                match_with=self.match_with,
+                recursive_match_all=self.recursive_match_all,
+                match_args=self.match_args,
+                match_kwargs=self.match_kwargs,
+            )
+        return rv
+
     def items(self, *args, **kwargs):
         # XXX what about when it doesn't exist?
         return super().items(*args, **kwargs)
 
     # Unique public methods
-    def paths_to_key(self, key):
+    def paths_to_key(self, key, *args, match_with=None, recursive_match_all=None, **kwargs):
         """
         >>> list(DeepCollection([{"x": {"y": "value", "z": {"y": "asdf"}}}]).paths_to_key("y"))
         [[0, 'x', 'y'], [0, 'x', 'z', 'y']]
         """
-        yield from paths_to_key(self, key)
+        match_with = match_with or self.match_with
+        match_args = args or self.match_args
+        match_kwargs = kwargs or self.match_kwargs
+        if recursive_match_all is None:
+            recursive_match_all = self.recursive_match_all
 
-    def values_for_key(self, key):
+        yield from paths_to_key(
+            self, key, *match_args, match_with=match_with, recursive_match_all=recursive_match_all, **match_kwargs
+        )
+
+    def values_for_key(self, key, *args, match_with=None, recursive_match_all=None, **kwargs):
         """
         >>> dc = DeepCollection([{"x": {"y": "v", "z": {"y": "v"}}, "y": {1: 2}}], return_deep=False)
         >>> list(dc.values_for_key("y"))
         ['v', 'v', {1: 2}]
         """
-        yield from values_for_key(self, key)
+        match_with = match_with or self.match_with
+        match_args = args or self.match_args
+        match_kwargs = kwargs or self.match_kwargs
+        if recursive_match_all is None:
+            recursive_match_all = self.recursive_match_all
 
-    def deduped_values_for_key(self, key):
+        yield from values_for_key(
+            self, key, *match_args, match_with=match_with, recursive_match_all=recursive_match_all, **match_kwargs
+        )
+
+    def deduped_values_for_key(self, key, *args, match_with=None, recursive_match_all=None, **kwargs):
         """
         >>> dc = DeepCollection([{"x": {"y": "v", "z": {"y": "v"}}, "y": {1: 2}}], return_deep=False)
         >>> 'v' in dc.deduped_values_for_key("y")  # order not gaurunteed
@@ -655,4 +838,21 @@ class DeepCollection(metaclass=DynamicSubclasser):
         >>> {1: 2} in dc.deduped_values_for_key("y")  # order not gaurunteed
         True
         """
-        return deduped_items(list(values_for_key(self, key)))
+        match_with = match_with or self.match_with
+        match_args = args or self.match_args
+        match_kwargs = kwargs or self.match_kwargs
+        if recursive_match_all is None:
+            recursive_match_all = self.recursive_match_all
+
+        return deduped_items(
+            list(
+                values_for_key(
+                    self,
+                    key,
+                    *match_args,
+                    match_with=match_with,
+                    recursive_match_all=recursive_match_all,
+                    **match_kwargs,
+                )
+            )
+        )
